function pkiwo(outFile,inCtl,maxShift)
%PKIWO Peak Increment, Write Out (CMU SSB Detector)
% PKIWO(outFile,inCtl,maxShift) generates a text file called outFile containing
% estimates of the SSB shift of each file specified in inCtl. That is,
% inCtl is a text file list of input files to be processed. outFile 
% contains the same number of lines as inCtl, each specifying the estimated
% SSB shift of the corresponding file. If the entry in the output list is
% 32768, this means that the algorithm was unable to accurately estimate the
% shift for that file (this is most likely to happen for a very noisy file
% of short duration).
%
% maxShift should be set equal to the absolute maximum shift necessary to be
% detected. Setting maxShift larger than necessary decreases the precision.
%
% outFile contains N rows, for N input files, and 4 columns. From left to right,
% the columns represent:
%  1. the estimated SSB shift
%  2. the confidence of the estimate (0.0 = least, 1.0 = most)
%  3. the fraction of time frames used to make the estimate
%  4. the average F0
%
% Input files should be either .wav or .sph uncompressed format. The file format
% is automatically deduced from the file extension. The sampling rate is assumed
% to be 16kHz.
%
% Contact:
% mharvill@cs.cmu.edu (Mark Harvilla)
% rms@cs.cmu.edu (Prof. Richard Stern)
%
% Written by Mark Harvilla
% Carnegie Mellon University
% September 10, 2011

tic;
T_wind   = 0.0256; %window length
T_shift  = 0.01;   %shift amount
iFFTSize = 2^13;   %number of DFT points

npeaks    = 4; %number of harmonics to consider, including the first peak
prc       = 75; %percentile of energy of frames for voicing decision cutoff
freq_sep  = 55; %minimum separation of peaks (in frequency)
mnpkht    = 0.1; %minimum peak height as a fraction of the maximum peak height
minFreq   = 55; %F0 minimum
maxFreq   = 300; %F0 maximum
%maxShift  = filtCutoff;

fs       = 16000;
%Wn       = filtCutoff/(fs/2); %discrete frequency cutoff
%filtOrd  = 500; %FIR filter order for narrow transition band (arbitrary)
%B        = fir1(filtOrd,Wn,'high'); %create filter

cnt = 0; %counter

fid    = fopen(inCtl,'r');
fidOut = fopen(outFile,'w');
while 1
 flag1 = 0;
 tline = fgetl(fid);
 if ~ischar(tline), break, end;

 fid2 = fopen(tline,'r');
 %[ad_x,fs] = wavread(tline); %read waveform
 %ad_x = filter(B,1,ad_x);
 if strcmp(tline(end-2:end),'wav') == 1 % <-- MS-WAV format
     fseek(fid2, 44, 'bof');
 else % <-- else assumed to be SPHERE format
     fseek(fid2, 1024, 'bof');
 end
 ad_x = fread(fid2, 'int16');
 ad_x = ad_x/32768;
 fclose(fid2);
 
 N1 = floor(T_wind*fs);  %window & hop size in points
 N2 = floor(T_shift*fs);
 w  = hamming(N1); %use a hamming window
 
 point_sep = ceil(freq_sep/(fs/iFFTSize)); %minimum separation of peaks (in points)

 iNumWindows = 1 + floor((length(ad_x) - N1)/N2); %determine number of windows

 %f_ax = 0:fs/iFFTSize:fs*(1-1/iFFTSize);
 %f_ax = -fs/2:fs/iFFTSize:(fs/2)-fs/iFFTSize;
 %f_ax = f_ax(1:(iFFTSize/2) + 1);

 thrsh          = 1;  %threshold for separating bad F0 estimates
 numValidFrames = 20; %initial minimum number of valid frames
 thrsh_max      = 10; %maximum F0 threshold allowed

 Frames_x   = buffer(ad_x,N1,N1-N2); %frame the signal
 Eframes_x1 = sum(repmat(w,1,size(Frames_x,2)).*(Frames_x.^2)); %find the energy in the frames
 Eframes_x2 = sum((repmat(w,1,size(Frames_x,2)).*Frames_x).^2); %alternate energy calculation
 pkCut      = prctile(Eframes_x1,prc); %determine the threshold value
 E_ind      = find(Eframes_x2 >= pkCut);
 f0         = zeros(npeaks,iNumWindows,thrsh_max - thrsh + 1);
 fp         = zeros(npeaks,iNumWindows,thrsh_max - thrsh + 1);
 n          = zeros(thrsh_max - thrsh + 1,1);
 firstDiff  = cell(thrsh_max - thrsh + 1,1);

 for q = 1:length(E_ind)
     x = Frames_x(:,E_ind(q)).*w;
     linSpec  = fft(x,iFFTSize); %spectrum
     linSpec  = abs(linSpec(1:iFFTSize/2 + 1)); %chop redundancy & drop phase
     [~,locs] = findpeaks(linSpec,'npeaks',npeaks,...
         'minpeakdistance',point_sep,'minpeakheight',...
         mnpkht*max(linSpec));
     if numel(locs) == npeaks
         for m = 1:npeaks
             X = [(locs(m)-1)^2,locs(m)-1,1;...
                 locs(m)^2,locs(m),1;...
                 (locs(m)+1)^2,locs(m)+1,1];
             Y = [linSpec(locs(m)-1);linSpec(locs(m));linSpec(locs(m)+1)];
             c = X\Y; 
             locs(m) = -c(2)/(2*c(1)); %interpolated index
         end
         f_loc = transpose((locs-1)*(fs/iFFTSize));
         for s = 1:thrsh_max - thrsh + 1
             firstDiff{s} = separateOutliers(diff(f_loc),thrsh+s-1);
             if numel(firstDiff{s}) == npeaks-1
             f0m = mean(firstDiff{s});
                 if f0m > minFreq && f0m < maxFreq
                     f0(:,n(s)+1,s) = f0m;
                     fp(:,n(s)+1,s) = f_loc;
                     n(s)           = n(s)+1;
                 end
             end
         end
     end
 end
 
 cnt = cnt + 1;
 if mod(cnt,100) == 0 
    fprintf('%d files processed\n',cnt);
 end
  
 while 1
     n1 = n;
     n2 = n;
     n1(n < numValidFrames) = [];
     n2(n < numValidFrames) = (1+iNumWindows);
     if isempty(n1) ~= 1
         [n_min,thrsh_ind] = min(n2);
         f0 = f0(:,1:n_min,thrsh_ind);
         fp = fp(:,1:n_min,thrsh_ind);
         f0 = f0(:);
         m  = fp(:);
         break;
     end
     %fprintf('reducing minimum valid frame requirement to %d\n',...
     %    numValidFrames-1);
     numValidFrames = numValidFrames - 1;
     if numValidFrames == 1
         %error('unable to process');
         fprintf(fidOut,'%f %f %f %f\n',32768,0,0,0);
         flag1 = 1;
     end
 end

 if flag1 ~= 1
  v  = length(m);

  fundShift = zeros(1,v-1);
  nums      = [];
  for q = 1:v-1
      fundShift(q) = mod(m(q),f0(q));
      nums1        = fundShift(q):f0(q):maxShift;
      nums2        = (fundShift(q)-f0(q)):-f0(q):-maxShift;
      nums         = [nums,nums1,nums2];
  end
  
  freqRes        = 5; %frequency resolution of estimate - for very low values, we probably need a lot of data
  %numBins        = ceil(2*maxShift/freqRes);
  binSpots       = -(maxShift-freqRes):freqRes:(maxShift-freqRes);
  Hout           = hist(nums,binSpots);
  %[Hout,binSpots] = hist(nums,numBins);
  [Hmax,HmaxInd] = max(Hout);
  HmaxFrac       = Hmax/sum(Hout);
  FrameFrac      = n_min/iNumWindows;
  fshift         = binSpots(HmaxInd);
  %fprintf('The frequency estimate is: %f\n',fshift);

  fprintf(fidOut,'%f %f %f %f\n',fshift,HmaxFrac,FrameFrac,mean(f0));
 end
end
fclose(fid);
fclose(fidOut);
toc;
